@@ -35,7 +35,8 @@ from azureml.train.hyperdrive import HyperDriveConfig, GridParameterSampling, Pr
 from azureml.dataprep.fuse.daemon import MountContext
 
 from health_azure.amulet import (ENV_AMLT_DATAREFERENCE_DATA, ENV_AMLT_DATAREFERENCE_OUTPUT, is_amulet_job)
-from health_azure.utils import (create_python_environment, create_run_recovery_id, find_file_in_parent_to_pythonpath,
+from health_azure.utils import (ENV_EXPERIMENT_NAME, create_python_environment, create_run_recovery_id,
+                                find_file_in_parent_to_pythonpath,
                                 is_run_and_child_runs_completed, is_running_in_azure_ml, register_environment,
                                 run_duration_string_to_seconds, to_azure_friendly_string, RUN_CONTEXT, get_workspace,
                                 PathOrString, DEFAULT_ENVIRONMENT_VARIABLES, get_ml_client,
@@ -408,6 +409,46 @@ def _generate_output_dataset_command(output_datasets_v2: Dict[str, Output]) -> s
     return output_cmd
 
 
+def get_display_name_v2(tags: Optional[Dict[str, Any]] = None) -> str:
+    """
+    If the command line argument 'tag' is provided, return its value to be set as the job's display name.
+    Empty spaces in the tag will be replaced with hyphens, otherwise AML treats it as multiple statements.
+    Otherwise return an empty string.
+
+    :param tags: An optional dictionary of tag names and values to be provided to the job.
+    :return: A string either containing the value of tag, or else empty.
+    """
+    if tags is None:
+        return ""
+    tag = tags.get("tag", "")
+    display_name = tag.replace(" ", "-")
+    return display_name
+
+
+def effective_experiment_name(experiment_name: Optional[str],
+                              entry_script: Optional[PathOrString] = None) -> str:
+    """Choose the experiment name to use for the run. If provided in the environment variable HIML_EXPERIMENT_NAME,
+    then use that. Otherwise, use the argument `experiment_name`, or fall back to the default based on the
+    entry point script.
+
+    :param experiment_name: The name of the AzureML experiment in which the run should be submitted.
+    :param entry_script: The script that should be run in AzureML.
+    :return: The effective experiment name to use, based on the fallback rules above.
+    """
+    value_from_env = os.environ.get(ENV_EXPERIMENT_NAME, "")
+    if value_from_env:
+        raw_value = value_from_env
+    elif experiment_name:
+        raw_value = experiment_name
+    elif entry_script is not None:
+        raw_value = Path(entry_script).stem
+    else:
+        raise ValueError("No experiment name provided, and no entry script provided. ")
+    cleaned_value = to_azure_friendly_string(raw_value)
+    assert cleaned_value is not None, "Expecting an actual string"
+    return cleaned_value
+
+
 def submit_run_v2(workspace: Optional[Workspace],
                   experiment_name: str,
                   environment: EnvironmentV2,
@@ -418,6 +459,7 @@ def submit_run_v2(workspace: Optional[Workspace],
                   script_params: Optional[List[str]] = None,
                   compute_target: Optional[str] = None,
                   tags: Optional[Dict[str, str]] = None,
+                  docker_shm_size: str = "",
                   wait_for_completion: bool = False,
                   wait_for_completion_show_output: bool = False,
                   workspace_config_path: Optional[PathOrString] = None,
@@ -440,6 +482,7 @@ def submit_run_v2(workspace: Optional[Workspace],
         locally.
     :param tags: A dictionary of string key/value pairs, that will be added as metadata to the run. If set to None,
         a default metadata field will be added that only contains the commandline arguments that started the run.
+    :param docker_shm_size: The Docker shared memory size that should be used when creating a new Docker image.
     :param wait_for_completion: If False (the default) return after the run is submitted to AzureML, otherwise wait for
         the completion of this run (if True).
     :param wait_for_completion_show_output: If wait_for_completion is True this parameter indicates whether to show the
@@ -484,6 +527,7 @@ def submit_run_v2(workspace: Optional[Workspace],
         output_datasets_v2 = {}
 
     job_to_submit: Union[Command, Sweep]
+    display_name = get_display_name_v2(tags)
 
     if hyperparam_args:
         param_sampling = hyperparam_args[PARAM_SAMPLING_ARG]
@@ -500,6 +544,9 @@ def submit_run_v2(workspace: Optional[Workspace],
             environment=environment.name + "@latest",
             compute=compute_target,
             experiment_name=experiment_name,
+            tags=tags or {},
+            shm_size=docker_shm_size,
+            display_name=display_name,
             environment_variables={
                 "JOB_EXECUTION_MODE": "Basic",
             }
@@ -530,6 +577,9 @@ def submit_run_v2(workspace: Optional[Workspace],
             environment=environment.name + "@latest",
             compute=compute_target,
             experiment_name=experiment_name,
+            tags=tags or {},
+            shm_size=docker_shm_size,
+            display_name=display_name,
             environment_variables={
                 "JOB_EXECUTION_MODE": "Basic",
             }
@@ -818,7 +868,6 @@ def submit_to_azure_if_needed(  # type: ignore
         snapshot_root_directory = Path.cwd()
 
     workspace = get_workspace(aml_workspace, workspace_config_path)
-    ml_client = get_ml_client(ml_client=ml_client, aml_workspace=workspace)
     print(f"Loaded AzureML workspace {workspace.name}")
 
     if conda_environment_file is None:
@@ -828,6 +877,7 @@ def submit_to_azure_if_needed(  # type: ignore
 
     amlignore_path = snapshot_root_directory / AML_IGNORE_FILE
     lines_to_append = [str(path) for path in (ignored_folders or [])]
+
     with append_to_amlignore(amlignore=amlignore_path, lines_to_append=lines_to_append):
         if strictly_aml_v1:
             run_config = create_run_configuration(
@@ -856,10 +906,8 @@ def submit_to_azure_if_needed(  # type: ignore
             else:
                 config_to_submit = script_run_config
 
-            effective_experiment_name = experiment_name or Path(script_run_config.script).stem
-
             run = submit_run(workspace=workspace,
-                             experiment_name=effective_experiment_name,
+                             experiment_name=effective_experiment_name(experiment_name, script_run_config.script),
                              script_run_config=config_to_submit,
                              tags=tags,
                              wait_for_completion=wait_for_completion,
@@ -874,8 +922,8 @@ def submit_to_azure_if_needed(  # type: ignore
             if entry_script is None:
                 entry_script = Path(sys.argv[0])
             script_params = script_params or sys.argv[1:]
-            effective_experiment_name = experiment_name or Path(entry_script).stem
 
+            ml_client = get_ml_client(ml_client=ml_client, aml_workspace=workspace)
             registered_env = register_environment_v2(environment, ml_client)
             input_datasets_v2 = create_v2_inputs(ml_client, cleaned_input_datasets)
             output_datasets_v2 = create_v2_outputs(cleaned_output_datasets)
@@ -883,13 +931,14 @@ def submit_to_azure_if_needed(  # type: ignore
             run = submit_run_v2(workspace=workspace,
                                 input_datasets_v2=input_datasets_v2,
                                 output_datasets_v2=output_datasets_v2,
-                                experiment_name=effective_experiment_name,
+                                experiment_name=effective_experiment_name(experiment_name, entry_script),
                                 environment=registered_env,
                                 snapshot_root_directory=snapshot_root_directory,
                                 entry_script=entry_script,
                                 script_params=script_params,
                                 compute_target=compute_cluster_name,
                                 tags=tags,
+                                docker_shm_size=docker_shm_size,
                                 wait_for_completion=wait_for_completion,
                                 wait_for_completion_show_output=wait_for_completion_show_output,
                                 hyperparam_args=hyperparam_args
